{{
Modified Tiny Basic for use with Propeller Demo Board, Hydra, C3, and others.
Full-Duplex Serial driver object based on fullDuplexSerial4port from the ObEx.

Portions copyright (c) 2008, 2013 Michael Green.
See end of file for terms of use.
}}

CON serPort = 0

OBJ ser : "fullDuplexSerial4port"

PUB start(rxpin, txpin, mode, baudrate) : okay

'' Start serial driver - starts a cog
'' returns false if no cog available
''
'' mode bit 0 = invert rx
'' mode bit 1 = invert tx
'' mode bit 2 = open-drain/source tx
'' mode bit 3 = ignore tx echo on rx

  ser.init
  ser.addPort(serPort, rxpin, txpin, ser#PINNOTUSED, ser#PINNOTUSED, {
    } ser#DEFAULTTHRESHOLD, mode, baudrate)
  okay := ser.start

PUB stop

'' Stop serial driver - frees a cog

  ser.stop

PUB rxflush

'' Flush receive buffer

  ser.rxflush(serPort)

PUB rxHowFull

'' Returns number of chars in rx buffer

  result := ser.rxHowFull(serPort)

PUB rxcheck : rxbyte

'' Check if byte received (never waits)
'' returns -1 if no byte received, $00..$FF if byte

  rxbyte := ser.rxcheck(serPort)

PUB rxtime(ms) : rxbyte | t

'' Wait ms milliseconds for a byte to be received
'' returns -1 if no byte received, $00..$FF if byte

  rxbyte := ser.rxtime(serPort, ms)  

PUB rx : rxbyte

'' Receive byte (may wait for byte)
'' returns $00..$FF

  repeat while (rxbyte := rxcheck) < 0


PUB txflush

'' Wait for transmit buffer to empty, then wait for byte to transmit

  ser.txflush(serPort)

PUB tx(txbyte)

'' Send byte (may wait for room in buffer)

  ser.tx(serPort, txbyte)

PUB str(stringptr)

'' Send string                    

  ser.str(serPort, stringptr)    

PUB strn(stringptr, nchar)

'' Send counted string

  ser.strn(serPort, stringptr, nchar)

PUB dec(value) | i

'' Print a decimal number

  if value < 0
    -value
    tx("-")

  i := 1_000_000_000

  repeat 10
    if value => i
      tx(value / i + "0")
      value //= i
      result~~
    elseif result or i == 1
      tx("0")
    i /= 10


PUB hex(value, digits)

'' Print a hexadecimal number

  value <<= (8 - digits) << 2
  repeat digits
    tx(lookupz((value <-= 4) & $F : "0".."9", "A".."F"))


PUB bin(value, digits)

'' Print a binary number

  value <<= 32 - digits
  repeat digits
    tx((value <-= 1) & 1 + "0")


PUB decn(value, digits) | mindigits, val

'' Print a decimal number, right-justified in a field width given by digits.

  mindigits := ((value < 0) & 1) + 1
  if (value <> $8000_0000)
    val := || value
    repeat while (val /= 10)
      mindigits++
  if (value < 0 and digits < 0)
    tx("-")
    value := ||value
  repeat (||digits - mindigits) #> 0
    if (digits < 0) 
      tx("0")
    else
      tx(" ")
  dec(value)

{{
                            TERMS OF USE: MIT License

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
}}
